@@ -18,6 +18,10 @@
 #        Author: Daniel Fett advancedcaching@fragcom.de
 #
 
+
+VERSION = 0
+VERSION_DATE = '2010-07-21'
+
 try:
     import json
     json.dumps
@@ -28,17 +32,19 @@ import datetime
 import geo
 import os
 import threading
+import logging
+logger = logging.getLogger('cachedownloader')
 global Image
 try:
     import Image
 except:
     Image = None
-    print "Not using image resize feature"
+    logger.info("Not using image resize feature")
 import re
 import gobject
 
 
-class HTMLManipulations:
+class HTMLManipulations(object):
     COMMENT_REGEX = re.compile('<!--.*?-->', re.DOTALL)
     
     @staticmethod
@@ -48,6 +54,16 @@ class HTMLManipulations:
         else:
             return re.sub(r'<[^>]*?>', ' ', text)
 
+    @staticmethod
+    def strip_html_visual(text, image_replace_callback):
+        text = text.replace("\n", " ")
+        text = re.sub(r"""(?i)<img[^>]+alt=["']?([^'"> ]+)[^>]+>""", image_replace_callback, text)
+        text = re.sub(r'(?i)<(br|p)[^>]*?>', "\n", text)
+        text = re.sub(r'<[^>]*?>', '', text)
+        text = HTMLManipulations._decode_htmlentities(text)
+        text = re.sub(r'[\n\r]+\s*[\n\r]+', '\n', text)
+        return text.strip()
+
     @staticmethod
     def _replace_br(text):
         return re.sub('<[bB][rR]\s*/?>|</?[pP]>', '\n', text)
@@ -118,8 +134,8 @@ class CacheDownloader(gobject.GObject):
         self.images = {}
 
     def _download_image(self, url):
-        print "+ Checking download for %s" % url
-        if url in self.downloaded_images.keys():
+        logger.info("Checking download for %s" % url)
+        if url in self.downloaded_images:
             return self.downloaded_images[url]
         
         ext = url.rsplit('.', 1)[1]
@@ -131,7 +147,7 @@ class CacheDownloader(gobject.GObject):
         if self.download_images:
             try:
                 filename = os.path.join(self.path, id)
-                print "+ Downloading %s to %s" % (url, filename)
+                logger.info("Downloading %s to %s" % (url, filename))
                 f = open(filename, 'wb')
                 f.write(self.downloader.get_reader(url).read())
                 f.close()
@@ -140,7 +156,7 @@ class CacheDownloader(gobject.GObject):
                     im.thumbnail((self.resize, self.resize), Image.ANTIALIAS)
                     im.save(filename)
             except Exception, e:
-                print "could not download %s: %s" % (url, e)
+                logger.exception("Could not download %s: %s" % (url, e))
                 return None
 
         self.downloaded_images[url] = id
@@ -151,7 +167,7 @@ class CacheDownloader(gobject.GObject):
         i = 0
         c = []
         if len(coordinates) > self.MAX_DOWNLOAD_NUM:
-            self.emit("download-error", Exception("Downloading of more than %d descriptions is not supported." % self.MAX_DOWNLOAD_NUM))
+            self.emit("download-error", Exception("Downloading of more than %d descriptions is not supported. (We do not want to knock out our beloved geocaching website.)" % self.MAX_DOWNLOAD_NUM))
             return
         for cache in coordinates:
             self.emit("progress", cache.name, i, len(coordinates))
@@ -161,21 +177,26 @@ class CacheDownloader(gobject.GObject):
         self.emit("finished-multiple", c)
         return c
                 
-    def update_coordinate(self, coordinate):
+    def update_coordinate(self, coordinate, outfile = None):
         if not CacheDownloader.lock.acquire(False):
             self.emit('already-downloading-error', Exception("There's a download in progress. Please wait."))
             return
         self._init_images()
         coordinate = coordinate.clone()
         self.current_cache = coordinate
-        #try:
-        print "* Downloading %s..." % (coordinate.name)
-        response = self._get_cache_page(coordinate.name)
-        u = self._parse_cache_page(response, coordinate)
-        #except Exception, e:
-        #    CacheDownloader.lock.release()
-        #    self.emit('download-error', e)
-        #    return self.current_cache
+        try:
+            logger.info("Downloading %s..." % (coordinate.name))
+            response = self._get_cache_page(coordinate.name)
+            if outfile != None:
+                f = open(outfile, 'w')
+                f.write(response.read())
+                f.close()
+                response = open(outfile)
+            u = self._parse_cache_page(response, coordinate)
+        except Exception, e:
+            CacheDownloader.lock.release()
+            self.emit('download-error', e)
+            return self.current_cache
         CacheDownloader.lock.release()
         self.emit('finished-single', u)
         return u
@@ -184,7 +205,7 @@ class CacheDownloader(gobject.GObject):
     def get_geocaches(self, location, rec_depth = 0):
         if not CacheDownloader.lock.acquire(False):
             self.emit('already-downloading-error', Exception("There's a download in progress. Please wait."))
-            print "downloading"
+            logger.warning("Download in progress")
             return
         # don't recurse indefinitely
         if rec_depth > self.MAX_REC_DEPTH:
@@ -202,6 +223,7 @@ class CacheDownloader(gobject.GObject):
         return points
 
 class GeocachingComCacheDownloader(CacheDownloader):
+    
     MAX_REC_DEPTH = 2
 
     MAX_DOWNLOAD_NUM = 20
@@ -217,6 +239,19 @@ class GeocachingComCacheDownloader(CacheDownloader):
         137:GeocacheCoordinate.TYPE_EARTH
     }
 
+    @staticmethod
+    def login_callback(username, password):
+        url = 'http://www.geocaching.com/Default.aspx'
+        values = {'ctl00$MiniProfile$loginUsername': username,
+            'ctl00$MiniProfile$loginPassword': password,
+            'ctl00$MiniProfile$uxRememberMe': 'on',
+            'ctl00$MiniProfile$LoginBtn': 'Go',
+            '__EVENTTARGET': '',
+            '__EVENTARGUMENT': ''
+        }
+        return url, values
+
+
     def _get_overview(self, location):
         c1, c2 = location
         url = 'http://www.geocaching.com/map/default.aspx?lat=49&lng=6'
@@ -244,8 +279,8 @@ class GeocachingComCacheDownloader(CacheDownloader):
         text = content.replace("\\'", "'")
         a = json.loads(text.replace('\t', ' '))
         points = []
-        if not 'cc' in a['cs'].keys():
-            if 'count' in a['cs'].keys() and 'count' != 0:
+        if not 'cc' in a['cs']:
+            if 'count' in a['cs'] and 'count' != 0:
                 # let's try to download one half of the geocaches first
                 mlat = (c1.lat + c2.lat)/2
                 nc1 = geo.Coordinate(min(c1.lat, c2.lat), min(c1.lon, c2.lon))
@@ -281,7 +316,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
     def _parse_cache_page(self, cache_page, coordinate):
         section = ''
         shortdesc = desc = hints = waypoints = images = logs = owner = head = ''
-        print "Start parsing"
+        logger.debug("Start parsing...")
         for line in cache_page:
             line = line.strip()
             
@@ -324,7 +359,8 @@ class GeocachingComCacheDownloader(CacheDownloader):
             elif section == 'images':
                 images = "%s%s " % (images, line)
 
-        print 'finished parsing'
+        logger.debug('finished parsing')
+
         coordinate.size, coordinate.difficulty, coordinate.terrain, coordinate.owner, coordinate.lat, coordinate.lon = self.__parse_head(head)
         coordinate.shortdesc = self.__treat_shortdesc(shortdesc)
         coordinate.desc = self.__treat_desc(desc)
@@ -349,7 +385,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
         elif sizestring == 'not_chosen' or sizestring == 'other':
             size = 5
         else:
-            print "Size not known: %s" % size
+            logger.warning("Size not known: %s" % size)
             size = 5
         diff = float(re.compile('(?s)Difficulty:</strong>.*?<img src="http://www.geocaching.com/images/stars/stars[0-9_]+\\.gif" alt="([0-9.]+) out').search(head).group(1))*10
         terr = float(re.compile('(?s)Terrain:</strong>.*?<img src="http://www.geocaching.com/images/stars/stars[0-9_]+\\.gif" alt="([0-9.]+) out').search(head).group(1))*10
@@ -357,7 +393,6 @@ class GeocachingComCacheDownloader(CacheDownloader):
         coords = re.compile('lat=([0-9.-]+)&amp;lon=([0-9.-]+)&amp;').search(head)
         lat = float(coords.group(1))
         lon = float(coords.group(2))
-        print size, diff, terr, owner, lat, lon
         return size, diff, terr, owner, lat, lon
         
         
@@ -413,7 +448,6 @@ class GeocachingComCacheDownloader(CacheDownloader):
             else:
                 comment = HTMLManipulations._decode_htmlentities(HTMLManipulations._strip_html(HTMLManipulations._replace_br(tds[3]))).strip()
                 waypoints.append({'lat':lat, 'lon':lon, 'id':id, 'name':name, 'comment':comment})
-        
         return waypoints
 
     def __treat_images(self, data):
@@ -430,21 +464,21 @@ class GeocachingComCacheDownloader(CacheDownloader):
         output = []
         for l in lines:
             #lines = [re.sub("\w+", ' ', HTMLManipulations._decode_htmlentities(HTMLManipulations._strip_html(x, True)), '').sub('[ view this log ]') for x in lines[2:]]
-            m = re.match(r"""<td[^>]+><strong><img src="http://www\.geocaching\.com/images/icons/icon_([a-z]+)\.gif"[^>]*/>""" +
-                r"""&nbsp;([^ ]+) (\d+)(, (\d+))? by <a[^>]+>([^<]+)</a></strong> \(\d+ found\)<br /><br />(.+)""" +
+            m = re.match(r"""<td[^>]+><strong><img src="http://www\.geocaching\.com/images/icons/(?:icon_(?P<icon>[a-z]+)|(?P<icon2>coord_update))\.gif"[^>]*/>""" +
+                r"""&nbsp;(?P<month>[^ ]+) (?P<day>\d+)(, (?P<year>\d+))? by <a[^>]+>(?P<finder>[^<]+)</a></strong> \(\d+ found\)<br /><br />(?P<text>.+)""" +
                 r"""<br /><br /><small>""", l, re.DOTALL)
             if m == None:
                 #print "Could not parse Log-Line:\nBEGIN\n%s\nEND\n\n This can be normal." % l
                 pass
             else:
-                type = m.group(1)
-                month = self.__month_to_number(m.group(2))
-                day = m.group(3)
-                year = m.group(5)
+                type = m.group('icon') if m.group('icon') != None else m.group('icon2')
+                month = self.__month_to_number(m.group('month'))
+                day = m.group('day')
+                year = m.group('year')
                 if year == '' or year == None:
                     year = datetime.datetime.now().year
-                finder = m.group(6)
-                text = HTMLManipulations._strip_html(HTMLManipulations._replace_br(m.group(7)), True)
+                finder = m.group('finder')
+                text = HTMLManipulations._strip_html(HTMLManipulations._replace_br(m.group('text')), True)
                 output.append(dict(type=type, month=month, day=day, year=year, finder=finder, text=text))
         return output
 
@@ -463,8 +497,8 @@ class GeocachingComCacheDownloader(CacheDownloader):
             return "[[img:%s]]" % id
 
     def __add_image(self, id, description = ''):
-        if ((id in self.images.keys() and len(description) > len(self.images[id]))
-            or id not in self.images.keys()):
+        if ((id in self.images and len(description) > len(self.images[id]))
+            or id not in self.images):
             self.images[id] = description
 
     @staticmethod
@@ -472,21 +506,23 @@ class GeocachingComCacheDownloader(CacheDownloader):
         months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
         if text in months:
             return months.index(text) + 1
-        print "Unknown month: " + text
+        logger.warning("Unknown month: " + text)
         return 0
 
 if __name__ == '__main__':
     import sys
-    name, password = sys.argv[1:3]
     import downloader
 
-    a = GeocachingComCacheDownloader(downloader.FileDownloader(name, password, '/tmp/cookies'), '/tmp/', True)
-    if len(sys.argv) < 3:
+    outfile = None
+    if len(sys.argv) == 2: # cachedownloder.py filename
+        print "Reading from file %s" % sys.argv[1]
         inp = open(sys.argv[1])
-        m = GeocacheCoordinate(0, 0, 'bla')
-        res = a._parse_cache_page(a, m)
-    else:
-        
+        m = GeocacheCoordinate(0, 0, 'GC1N8G6')
+        a = GeocachingComCacheDownloader(downloader.FileDownloader('dummy', 'dummy', '/tmp/cookies'), '/tmp/', True)
+    else: # cachedownloader.py username password
+        name, password = sys.argv[1:3]
+        a = GeocachingComCacheDownloader(downloader.FileDownloader(name, password, '/tmp/cookies'), '/tmp/', True)
+
         print "Using Username %s" % name
 
         def pcache(c):
@@ -505,9 +541,12 @@ if __name__ == '__main__':
             
         else:
             print "# Didn't find my own geocache :-("
-    res = a.update_coordinates([m])
+        if len(sys.argv) == 4:
+            print "Writing to File %s" % sys.argv[3]
+            outfile = sys.argv[3]
+    res = a.update_coordinate(m, outfile)
     print res
-    c = res[0]
+    c = res
     if c.owner != 'webhamster':
         print "Owner doesn't match ('%s', expected webhamster)" % c.owner
     if c.title != 'Druidenpfad':
