@@ -17,19 +17,24 @@
 #
 #        Author: Daniel Fett advancedcaching@fragcom.de
 #
-from socket import setdefaulttimeout
-from cookielib import LWPCookieJar
-from urllib import urlencode
-import urllib2
-setdefaulttimeout(30)
+
+
+import logging
+logger = logging.getLogger('downloader')
+
 class FileDownloader():
     USER_AGENT = 'User-Agent: Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.0.12) Gecko/2009070811  Windows NT Firefox/3.1'
+    opener_installed = False
 
-    def __init__(self, username, password, cookiefile):
+    def __init__(self, username, password, cookiefile, login_callback):
         self.username = username
         self.password = password
         self.cookiefile = cookiefile
         self.logged_in = False
+        from socket import setdefaulttimeout
+        setdefaulttimeout(30)
+        self.opener_installed = False
+        self.login_callback = login_callback
 
     def update_userdata(self, username, password):
         from os import path, remove
@@ -40,81 +45,87 @@ class FileDownloader():
             try:
                 remove(self.cookiefile)
             except:
-                print "* Could not remove cookie file?!"
+                logger.info("Could not remove cookie file?!")
                 pass
 
 
     def login(self):
         if self.username == '' or self.password == '':
             raise Exception("Please configure your username/password and restart the application")
-        print "+ Checking Login status"
+        logger.info("Checking Login status")
+        from cookielib import LWPCookieJar
         cj = LWPCookieJar(self.cookiefile)
-        opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
-        urllib2.install_opener(opener)
+        if not self.opener_installed:
+            from urllib2 import build_opener, install_opener, HTTPCookieProcessor
+            opener = build_opener(HTTPCookieProcessor(cj))
+            install_opener(opener)
+            self.opener_installed = True
 
         try:
             cj.load()
-            print "+ Loaded cookie file"
+            logger.info("Loaded cookie file")
         except:
-            print "+ Couldn't load cookie file"
+            logger.info("Couldn't load cookie file")
         else:
-            print "+ Checking if still logged in..."
+            logger.info("Checking if still logged in...")
             url = 'http://www.geocaching.com/seek/nearest.aspx'
             page = self.get_reader(url, login = False)
             for line in page:
                 if 'You are logged in as' in line:
                     self.logged_in = True
-                    print "+ Seems as we're still logged in"
+                    logger.info("Seems as we're still logged in")
                     return
                 elif 'You are not logged in.' in line:
-                    print "+ Nope, not logged in anymore"
+                    logger.info("Nope, not logged in anymore")
                     break
         
-        print "+ Logging in"
-        url = 'http://www.geocaching.com/Default.aspx'
-        values = {'ctl00$MiniProfile$loginUsername':self.username,
-            'ctl00$MiniProfile$loginPassword':self.password,
-            'ctl00$MiniProfile$uxRememberMe': 'on',
-            'ctl00$MiniProfile$LoginBtn': 'Go',
-            '__EVENTTARGET': '',
-            '__EVENTARGUMENT': ''
-        }
+        logger.info("Logging in")
+        url, values = self.login_callback(self.username, self.password)
 
-        page = self.get_reader(url, values, login = False).read()
+        page = self.get_reader(url, values, login = False)
 
-        if 'combination does not match' in page:
-            raise Exception("Wrong password or username!")
-        print "+ Great success."
+        for line in page:
+            if 'You are logged in as' in line:
+                break
+            elif 'You are not logged in.' in line or 'combination does not match' in line:
+                raise Exception("Wrong password or username!")
+        else:
+            logger.info("Seems as if the language is set to something other than english")
+            raise Exception("Please go to geocaching.com and set the website language to english!")
+
+        logger.info("Great success.")
         self.logged_in = True
         try:
             cj.save()
         except Exception, e:
-            print "+ Could not save cookies:", e
+            logger.info("Could not save cookies: %s" % e)
 
 
     def get_reader(self, url, values=None, data=None, login = True):
+        from urllib import urlencode
+        from urllib2 import Request, urlopen
         if login and not self.logged_in:
             self.login()
 
         if values == None and data == None:
-            req = urllib2.Request(url)
+            req = Request(url)
             self.add_headers(req)
-            return urllib2.urlopen(req)
+            return urlopen(req)
 
         elif data == None:
             if (isinstance(values, dict)):
                 values = urlencode( values)
-            req = urllib2.Request(url, values)
+            req = Request(url, values)
             self.add_headers(req)
-            return urllib2.urlopen(req)
+            return urlopen(req)
         elif values == None:
             content_type, body = data
-            req = urllib2.Request(url)
+            req = Request(url)
             req.add_header('Content-Type', content_type)
             req.add_header('Content-Length', len(str(body)))
             self.add_headers(req)
             req.add_data(body)
-            return urllib2.urlopen(req)
+            return urlopen(req)
 
     def encode_multipart_formdata(self, fields, files):
         """
