@@ -18,9 +18,14 @@
 #        Author: Daniel Fett advancedcaching@fragcom.de
 #
 
+VERSION = 0
+VERSION_DATE = '2010-07-03'
+
 import geocaching
 import re
 import gobject
+import logging
+logger = logging.getLogger('fieldnotesuploader')
 
 
 class FieldnotesUploader(gobject.GObject):
@@ -38,25 +43,25 @@ class FieldnotesUploader(gobject.GObject):
         self.notes = []
 
     def add_fieldnote(self, geocache):
-        if geocache.log_date == '':
+        if geocache.logdate == '':
             raise Exception("Illegal Date.")
 
-        if geocache.log_as == geocaching.GeocacheCoordinate.LOG_AS_FOUND:
+        if geocache.logas == geocaching.GeocacheCoordinate.LOG_AS_FOUND:
             log = "Found it"
-        elif geocache.log_as == geocaching.GeocacheCoordinate.LOG_AS_NOTFOUND:
+        elif geocache.logas == geocaching.GeocacheCoordinate.LOG_AS_NOTFOUND:
             log = "Didn't find it"
-        elif geocache.log_as == geocaching.GeocacheCoordinate.LOG_AS_NOTE:
+        elif geocache.logas == geocaching.GeocacheCoordinate.LOG_AS_NOTE:
             log = "Write note"
         else:
-            raise Exception("Illegal status: %s" % self.log_as)
+            raise Exception("Illegal status: %s" % geocache.logas)
 
         text = geocache.fieldnotes.replace('"', "'")
 
-        self.notes.append('%s,%sT10:00Z,%s,"%s"' % (geocache.name, geocache.log_date, log, text))
+        self.notes.append('%s,%sT10:00Z,%s,"%s"' % (geocache.name, geocache.logdate, log, text))
 
     def upload(self):
         try:
-            print "+ Uploading fieldnotes..."
+            logger.info("Uploading fieldnotes...")
             page = self.downloader.get_reader(self.URL).read()
             m = re.search('<input type="hidden" name="__VIEWSTATE" id="__VIEWSTATE" value="([^"]+)" />', page)
             if m == None:
@@ -74,5 +79,6 @@ class FieldnotesUploader(gobject.GObject):
                 raise Exception("Something went wrong while uploading the field notes.")
             else:
                 self.emit('finished-uploading')
+                logger.info("Finished upload")
         except Exception, e:
             self.emit('upload-error', e)
